#
# vim:set syntax=python:

import java 
import sys
import time
import re

lineSeparator = java.lang.System.getProperty('line.separator') 

#  ~~~~~                                                          ~     ~    ~     ~~~~~  
# ~     ~  ~~~~~~  ~    ~  ~~~~~~  ~~~~~      ~     ~~~~          ~  ~  ~   ~ ~   ~     ~ 
# ~        ~       ~~   ~  ~       ~    ~     ~    ~    ~         ~  ~  ~  ~   ~  ~       
# ~  ~~~~  ~~~~~   ~ ~  ~  ~~~~~   ~    ~     ~    ~              ~  ~  ~ ~     ~  ~~~~~  
# ~     ~  ~       ~  ~ ~  ~       ~~~~~      ~    ~              ~  ~  ~ ~~~~~~~       ~ 
# ~     ~  ~       ~   ~~  ~       ~   ~      ~    ~    ~         ~  ~  ~ ~     ~ ~     ~ 
#  ~~~~~   ~~~~~~  ~    ~  ~~~~~~  ~    ~     ~     ~~~~           ~~ ~~  ~     ~  ~~~~~  
#
#
#--------------------------------------------------------
#                                                                                         
# Retrieve and Store the Cell Information
#
#--------------------------------------------------------
def getCellAndNodeInfo(cfg):
	
	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
	except: 
		print "Can Load the wsadmin Modules"

        #print "Initial Load: Cell information....",

	cellName  = AdminControl.getCell()
	cellID    = AdminConfig.getid('/Cell:' + cellName + '/')
	cellStr   = '/Cell:' + cellName + '/'

	list      = []
	list.append({'name':cellName, 'ID':cellID})
	cfg.cell = list
	
	# construct the node information
	list      = []
	for node in AdminConfig.list('Node').split('\n'):
	
	     nodeName = AdminConfig.showAttribute(node, 'name')
	     nodeID   = AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName + '/')
	     list.append({'name':nodeName, 'ID':nodeID, 'cellName':cellName, 'cellID':cellID})

	cfg.nodes = list
	
        #print "Done"

	return


#--------------------------------------------------------
#
# Retrieve and Store Info on all Servers.
# Also generate Introscope stats for the Server definition and Check standards
#
#--------------------------------------------------------
def getServerInfo(cfg):

	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
	except: 
		print "Can Load the wsadmin Modules"


	skipMgmtSrvs = 0  # set to 1 and servers named dmgr and nodeagent are ignored
	
	cellName  = AdminControl.getCell()
	cellID    = AdminConfig.getid('/Cell:' + cellName + '/')
	cellStr   = '/Cell:' + cellName + '/'

	list      = []

	for node in AdminConfig.list('Node').split('\n'):
	
		nodeName = AdminConfig.showAttribute(node, 'name')
		nodeID   = AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName + '/')

		# Iterate through each server in 'node'

		for server in AdminConfig.list('Server', node).split('\n'):
		
			serverName  = AdminConfig.showAttribute( server, 'name')
			clusterName = AdminConfig.showAttribute( server, 'clusterName')
			
			if clusterName == None:
				clusterName = 'AA-Stand-Alone'
				clusterID   = 'AA-Stand-Alone'
			else:
				clusterID   = AdminConfig.getid( '/ServerCluster:%s/' % clusterName )
			
			serverID = AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName + '/Server:' + serverName + '/')
			
			if skipMgmtSrvs == 1 and serverName in ['dmgr', 'nodeagent']: continue

			# construct the server information

			list.append({'name'       :serverName, 
			             'ID'         :serverID,
			             'clusterName':clusterName, 
			             'clusterID'  :clusterID,
                                     'nodeName'   :nodeName,
                                     'nodeID'     :nodeID,
                                     'cellName'   :cellName,
                                     'cellID'     :cellID,
                                    })
                        
                        

        cfg.servers = list
        
        #print "Done"

	return

#--------------------------------------------------------
#
# Get the DataSource information  
#
#--------------------------------------------------------
def getDataSourceInfo(cfg):

	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
	except: 
		print "Can Load the wsadmin Modules"

        #print "Initial Load: DataSource information....",

	skipMgmtSrvs = 0  # set to 1 and servers named dmgr and nodeagent are ignored
	
	cellName  = AdminControl.getCell()
	cellID    = AdminConfig.getid('/Cell:' + cellName + '/')
	cellStr   = '/Cell:' + cellName + '/'

	list = {'cell':[], 'clusters':[], 'nodes':[], 'servers':[]   }
		
	for dataSource in AdminConfig.list('DataSource').splitlines():
	
		name = AdminConfig.showAttribute(dataSource, 'name')
		start  = dataSource.find('(')
		end    = dataSource.find('|')

                #	print "EEEEeeeekkk! === %s" % dataSource[start + 1:end] 

		heir   = dataSource[start + 1:end].split('/')
		cell = ''
		cluster = ''
		node = ''
		server = ''

                #
                # Find the Scope of the Datasource
                #
		for iter in range(0, len(heir)):

			if heir[ iter ] == 'cells':
				cell = heir[ iter + 1 ]
				scope = 'cell'

			elif heir[ iter ] == 'clusters':
				cluster = heir[ iter + 1 ]
				scope = 'cluster'

			elif heir[ iter ] == 'nodes':
				node = heir[ iter + 1 ]
				scope = 'node'

			elif heir[ iter ] == 'servers':
				server = heir[ iter + 1 ]
				scope = 'server'

                #	print "name %s" % name 
                #	print "ID %s" % dataSource 
                # 	print "scope %s" % scope 

		if scope == "cell"    : list[ 'cell' ].append({'cell':cell, 'name':name,'metric':cell, 'ID':dataSource })
		if scope == "cluster" : list[ 'clusters' ].append({'cell':cell, 'cluster':cluster, 'name':name,'metric':cell + '|' + cluster, 'ID':dataSource})
		if scope == "nodes"   : list[ 'nodes' ][ nodes ].append({'cell':cell, 'node':node, 'name':name,'metric':cell + '|' + node, 'ID':dataSource})
		if scope == "servers" : list[ 'servers' ][ servers ].append({'cell':cell, 'node':node, 'server':server, 'name':name,'metric':server, 'ID':dataSource })

        cfg.dataSources = list

        #print "Done"



# - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - #


def getObjectName(cfg, ID):
	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
		AdminApp     = cfg.AdminApp
		AdminTask    = cfg.AdminTask
	except: 
		print "Can Load the wsadmin Modules"

	return AdminConfig.getObjectName(ID)

# - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - # - = - #


def getObjectConfig(cfg, object):
	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
		AdminApp     = cfg.AdminApp
		AdminTask    = cfg.AdminTask
	except: 
		print "Can Load the wsadmin Modules"

	print "Server Object is: %s" % object
# 	list = AdminConfig.showall(object) 
	print "--------------------------------------------------------------------------------------------------------"
# 	print "HEre size : %s" % list[1] 
# 	print list 
	result = {}; 
# 	aDict = {}  

	listNumber = 0
	heirarchy = ""
	for item in AdminConfig.showall(object).splitlines(): 
# 	item = re.sub('\]\\n\[', '][', AdminConfig.showall(object)) 
# 	print item 
#  	item = item[1:len(item) - 1]  
#  	spacePos = item.find(' ')  
# 	theName  = item[0:spacePos] 
# 	theValue = item[spacePos + 1:]  
# 	pos      = spacePos + 1  
# 	[ theName, theValue, heirarchy, listNumber, newHeirarchy ] = complexListNext(theName, theValue, heirarchy, listNumber)   
#  	print '%s%s = %s' % (heirarchy, theName, theValue) 
# 	quit() 
# 		print "------------------------------------------------------------------------------------------------------" 
# 		print "%s" % item 
		item = item[1:len(item) - 1] 
		spacePos = item.find(' ') 
		theName  = item[0:spacePos]
		theValue = item[spacePos + 1:] 
		pos      = spacePos + 1 
# 	print 'Before Name: Heir(%s) name(%s) = val(%s)' % (heirarchy, theName, theValue) 
		if len(theValue) > 1:
			if theValue[1] == "[": 
# 			print "line 75" 
				[ theName, theValue, heirarchy, listNumber, newHeirarchy ] = complexListNext(cfg, theName, theValue, heirarchy, listNumber)  
# 			print 
# 			print 'After  Name: %s%s = %s' % (heirarchy, theName, theValue) 
# 			print 'After  Heirarchy: %s' % newHeirarchy 
			if theValue[-1] == "]": 
				[ theValue, newHeirarchy ] = reduceHeirarchy(cfg, theValue, heirarchy)  
			result[heirarchy + theName] = theValue
#  	print '       Name: %s%s = %s' % (heirarchy, theName, theValue) 
# 	print '       Heirarchy: %s' % newHeirarchy 
 	print '%s%s = %s' % (heirarchy, theName, theValue)
	heirarchy = newHeirarchy
	exit

def complexListNext(cfg, theName, theValue, heirarchy, listNumber):
	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
		AdminApp     = cfg.AdminApp
		AdminTask    = cfg.AdminTask
	except: 
		print "Can Load the wsadmin Modules"

# 	print 
# 	print "     ---Complex" 
# 	print "     ENTERING DEBUG Heir(%s) Name(%s) val(%s) list(%s)" % (heirarchy, theName, theValue, listNumber)    
	heirarchy = heirarchy + theName + "."
	newHeirarchy = heirarchy
	theName = "list" + str(listNumber)
	listNumber = listNumber + 1
	theValue  = theValue[1:]
	if theValue[1] == "[": 
		theName = "list" + str(listNumber)
		listNumber = listNumber + 1
# 		print "line 100" 
		[ theName, theValue, heirarchy, listNumber, newHeirarchy ] = complexListNext(cfg, theName, theValue, heirarchy, listNumber)  
	if theValue[-1] == "]": 
# 		print "line 103" 
		[ theValue, newHeirarchy ] = reduceHeirarchy(cfg, theValue, heirarchy)  
	if theValue[0] == "[": 
		if theValue[1] == "[": 
# 			theName = "list" + str(listNumber) 
# 			listNumber = listNumber + 1 
# 			print "line 109" 
			[ theName, theValue, heirarchy, listNumber, newHeirarchy ] = complexListNext(cfg, theName, theValue, heirarchy, listNumber)  
	if theValue[0] == "[": 
		spacePos  = theValue.find(' ')  
		heirarchy = heirarchy + theName + "." 
		theName   = theValue[1:spacePos] 
		theValue  = theValue[spacePos + 1:]  
# 		if theValue[1] == "[":  
# 			listNumber = listNumber + 1 
# 			theName = "list" #% str(listNumber) 
# 			print "DEBUG %s %s %s %s" % (theName, theValue, heirarchy, listNumber)   
# 			[ theName, theValue, heirarchy, listNumber, newHeirarchy ] = complexListNext(theName, theValue, heirarchy, listNumber)   
# 	print "     EXITRING DEBUG Heir(%s) Name(%s) Val(%s) list(%s)" % (heirarchy, theName, theValue, listNumber)    
	return [ theName, theValue, heirarchy, listNumber, newHeirarchy ]

def reduceHeirarchy(cfg, theValue, heirarchy):
	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
		AdminApp     = cfg.AdminApp
		AdminTask    = cfg.AdminTask
	except: 
		print "Can Load the wsadmin Modules"

# 	print "     ---Reduce" 
# 	print "     ENTERING DEBUG Heir(%s) val(%s)" % (heirarchy, theValue)    
	if theValue <> "[]":
		theValue = theValue[:len(theValue) - 1]
		scopeRegExp = re.compile(r'[^.]*.$')
# 		print "reduce heirarchy: %s" % heirarchy 
		heirarchy = scopeRegExp.sub(r'', heirarchy)
# 		print "reduce heirarchy: %s" % heirarchy 
		if theValue[-1] == "]": 
			[ theValue, heirarchy ] = reduceHeirarchy(cfg, theValue, heirarchy)  
# 	print "     EXITRING DEBUG Heir(%s) val(%s)" % (heirarchy, theValue)    
	return [ theValue, heirarchy ]
# 	aDict 
# {'internalClassAccessMode': 'ALLOW', 'debugArgs': '"-Djava.compiler=NONE -Xdebug -Xnoagent -Xrunjdwp:transport=dt_socket,server=y,suspend=n,address=7777"', 'classpath': '[]', 'initialHeapSize': '256', 'runHProf': 'false', 'genericJvmArguments': '[]]', 'hprofArguments': '[]]', 'bootClasspath': '[]', 'verboseModeJNI': 'false', 'systemProperties': '[myname(cells/wasCell/nodes/appServNode/servers/engine1|server.xml#Property_1191570369046)]', 'maximumHeapSize': '512', 'disableJIT': 'false', 'verboseModeGarbageCollection': 'true', 'verboseModeClass': 'false','debugMode': 'false'} 

# 	aDict["maximumHeapSize"] 
# '512' 
# 	aDict["verboseModeGarbageCollection"] 
# 'true' 


# 	for item in list.splitlines() : 
# 		if ( item[ 0 ] == '[' ) and ( item[ -1 ] == ']' ) : 
# 			( name, value ) = item[ 1:-1 ].split( ' ', 1 ); 
# 			if re.match('^\[.*\]$', value):   
# 				print AdminConfig.showAttribute(name) 
# 				print "%s ---------- %s" % name, dictify(value) 
# 				value=getObjectConfig(AdminConfig.getid(name))   
# 			print '   name="%s"  value="%s"' % ( name, value ); 

# 			result[ name ] = value; 
	# print result['retriesCount'];
# 	return AdminConfig.showall(object) 
	
def dictify(cfg, value):
	try:
		AdminConfig  = cfg.AdminConfig
		AdminControl = cfg.AdminControl
		AdminApp     = cfg.AdminApp
		AdminTask    = cfg.AdminTask
	except: 
		print "Can Load the wsadmin Modules"

	print 'DicTiFy!!'
	return
