#
# vim:set syntax=python:
#import os

import sys
import time
from types import *


#   ---------=========== CLASS: log         ===========--------- 


class log:                           # Log class -- Does the logging work
	def __init__(self, file, level = 5): # Instanciate log obj. by passing logfile name, and/or log lvl 
		                                   #   log lvl is 0-7, 0 is ALERT, 7 is Debug (syslog lvls)
		self.filename  = file              # log file name
		self.level     = level             # log lvl
# 		self.cfg       = cfg               # So I can find config later 

		self.lf = open(self.filename,'a',-1)
# 		self.lf.write("Process started\n") 
		self.startDate  = time.asctime( time.localtime(time.time()) ) # Start time of prog
		self.write("---=== Log started " + self.startDate + " ===---\n") 

	def close(self):
		self.lf.flush()
		self.lf.close()

	def write(self,msg):
# 		self.lf.write(msg.rstrip() + '\n') 
		print >>self.lf, msg.rstrip()

	# log classes. (syntax e.g. log.debug("blip blop flip flop))

	def debug(self,msg):
		if self.level >= 7: self.write(msg)

	def info(self,msg):
		if self.level >= 6: self.write(msg)

	def notice(self,msg):
		if self.level >= 5: self.write(msg)

	def warn(self,msg):
		if self.level >= 4: self.write(msg)

	def error(self,msg):
		if self.level >= 3: self.write(msg)

	def critical(self,msg):
		if self.level >= 2: self.write(msg)

	def alert(self,msg):
		if self.level >= 1: self.write(msg)

	def emergency(self,msg):
		if self.level >= 0: self.write(msg)


#   ---------=========== CLASS: introscope  ===========---------


class introscope:                           # Introscope class -- provides methods for introscope stuff

	def __init__(self, cfg, outStream = 'stdout', outFmt = 'xml'): # Instanciate instroscope obj, outStream = (stdout|socket), outFmt = (xml|simple)
		self.outFmt    = outFmt                                      #   Using socket requires the cfg object to hold the necessary
		self.outStream = outStream 
		self.cfg       = cfg                                         # So I can find config later
		self.metricPrefix = ""                                       # So I can find config later
		
	def prefix(self, value):                                       # I think that heirarchy will be a turple
		# Possible syntaxs are:
		#   obj.prefix( 'WASMonitor' )
		#   obj.prefix( 'WASMonitor|stuff|1|2' )
		#   obj.prefix([ "WASMonitor", "fred", "ginger", "apple" ])
		nameString = '' 
                                                                 # This chunk looks at the value param
		if type(value) == ListType:                                   # LIST: is the val of value a list ?
			if len(value) > 1:                                          # LIST: is the list in value more than 1 item
				first = 1
				for index in range(len(value) - 1):                       # LIST: print a item except last one, prefix with '|' except the first one
					if first == 1:  
						first = 0 
						nameString += value[index] 
					else: 
						nameString += '|' + value[index]                      
				nameString += '|' + value[-1]                             # LIST: print last item with a ':' prefix
			else: 
				nameString += value[-1]                                   # LIST: print the 1 list item as if it was a preprepared string
		else: 
			nameString += value                                         # STRING: print the preprepared string
		self.metricPrefix = nameString + "|"
		print "Introscope Metrics have a new common prefix: %s" % self.metricPrefix

	def metric(self, metricType, name, value):                     # I think that heirarchy will be a turple
		# Possible syntaxs are:
		# 	obj.metric("IntCounter", ['one', 'two', 'three'], "valuealicious")  
		# 	obj.metric("IntCounter", ['one|two:three'],       "valuealicious")  
		# 	obj.metric("IntCounter", 'one|two:three',         "valuealicious")  
		# -------- Metric Types -------
		metricTypes = [ 'IntAverage',         \
                    'IntCounter',         \
                    'LongCounter',        \
                    'PerIntervalCounter', \
                    'StringEvent' ]                              # metricTypes will be used in the furture, where metricType can be numeric
		nameString = self.metricPrefix
                                                                 # This chunk looks at the name param
		if type(name) == ListType:                                   # LIST: is the val of name a list ?
			if len(name) > 1:                                          # LIST: is the list in name more than 1 item
				first = 1
				for index in range(len(name) - 1):                       # LIST: print a item except last one, prefix with '|' except the first one
					if first == 1:  
						first = 0 
						nameString += name[index] 
					else: 
						nameString += '|' + name[index]                      
				nameString += ':' + name[-1]                             # LIST: print last item with a ':' prefix
			else: 
				nameString += name[-1]                                   # LIST: print the 1 list item as if it was a preprepared string
		else: 
			nameString += name                                         # STRING: print the preprepared string

		print '<metric type="' + metricType + '" name="' + nameString + '" value="' + value + '" />'
